<?php

if (!defined('PLX_ROOT')) exit;

$fields = array(
	'nb_pages'		=> 'numeric',
	'category'		=> 'boolean',
	'tags'			=> 'boolean',
	'no_tags'		=> 'string',
	'template'		=> 'string',
	'archive'		=> 'string',
	'author'		=> 'string',
);
$default_values = array(
	'nb_pages'		=> 5,
	'category'		=> true,
	'tags'			=> true,
	'no_tags'		=> 'a,à,au,ayant,celles,contre,dans,d,de,des,du,devant,en,entre,et,l,le,la,les,on,par,pour,que,qui,s,sa,se,ses,son,un,une',
);

$aTemplates = array();
$files = plxGlob::getInstance(PLX_ROOT . $plxAdmin->aConf['racine_themes'] . $plxAdmin->aConf['style']);
if ($array = $files->query('#^article(-[\w-]+)?.php$#')) {
	foreach($array as $k=>$v) {
		$aTemplates[$v] = $v;
	}
}

$aAuthors = array();
foreach ($plxAdmin->aUsers as $id=>$infos) {
	if (!empty($infos['active']) and empty($infos['delete'])) {
		$aAuthors[$id] = $infos['name'];
	}
}

function getTags($aString, $enabled=true) {
	global $plxPlugin;

	if ($enabled) {
		// catching keywords in $aString
		$noTags = explode(',', $plxPlugin->getParam('no_tags'));
		$buf1 = strtolower(str_replace(array('Â','Ê','É'), array('â', 'ê', 'é'), $aString));
		if (preg_match_all('#(^|[^\wàâéèêîôùû])*([\wàâéèêîôùû]+)([^\wàâéèêîôùû]|$)*#', $buf1, $buf)) {
			$result = $buf[2];
			for ($i = count($result) - 1; $i >= 0; $i--) {
				if (in_array($result[$i], $noTags))
					unset($result[$i]);
			}
			return implode(',', array_unique($result));
		}
		else
			return '';
	}
	else
		return '';
}

function addCat($newCat, $add=true) {
	global $plxAdmin;

	$result = '000'; // It's default category
	if ($add) {
		// checking if $newCat is registered
		$notFound = true;
		foreach (array_keys($plxAdmin->aCats) as $k) {
			if ($plxAdmin->aCats[$k]['name'] == $newCat) {
				$notFound = false;
				$result = $k;
				break;
			}
		}
		if ($notFound) {
			// Really, $newCat is new
			$content = array('new_category'=>1, 'new_catname'=>$newCat);
			$plxAdmin->editCategories($content);
			$keys = array_keys($plxAdmin->aCats);
			$result = $keys[count($keys) - 1];
		}
	}
	return $result;
}

function getSrcDispo($zipfile) {
	global $plxAdmin;

	# urls déjà utilisés pour les articles
	$urls = array_map(
		function($filename) {
			return preg_replace('#^[\d\.]*\.(.*)\.xml$#', '$1', $filename);
		},
		$plxAdmin->plxGlob_arts->aFiles
	);


	$lignes = explode("\n", $zipfile->getFromName('index.csv'));
	$result = array();
	foreach ($lignes as $ligne) {
		$buf = trim($ligne);
		if(!empty($buf)) {
			list($livre, $title, $src) = explode("\t", $ligne);
			$src = trim($src);
			if (!empty($src))
				$result[$src] = array('title'=>$title, 'cat'=>$livre);
		}
	}
	ksort($result);

	# pas de doublons pour les urls
	return array_filter(
		$result,
		function($value, $url) use($urls) {
			return !in_array($url, $urls);
		},
		ARRAY_FILTER_USE_BOTH
	);
}

function readArticles($filename, $nb_articles) {
	if(!class_exists('ZipArchive')) {
		return;
	}

	$zipfile = new ZipArchive();
	if ($zipfile->open($filename)) {
		global $plxAdmin, $plxPlugin, $aAuthors, $aTemplates;

		$enabledTags = !empty($plxPlugin->getParam('tags'));
		$enabledCats = !empty($plxPlugin->getParam('category'));
		$author = $plxPlugin->getParam('author');
		$template = $plxPlugin->getParam('template');

		$liste = getSrcDispo($zipfile);
		$nextIdArticle = $plxAdmin->nextIdArticle();
		for ($i = 0; $i < $nb_articles; $i++) {
			if (count($liste) > 0) {
				mt_srand();
				$k = mt_rand(0, count($liste) - 1);
				$keys = array_keys($liste);
				$src = $keys[$k];
				$content = $zipfile->getFromName($src.'.inc');
				$temp = explode("\n", trim($content));
				$n = mt_rand(3, 7);
				$lines = (count($temp) > $n + 2) ? $n : count($temp) - 2; // balise <p> sur 1ère et dernière ligne
				$chapo = array_splice($temp, 1, $lines);
				$x = count($chapo) - 1;
				$chapo[$x] = preg_replace('#\s*<br\s*/>\s*$#', '', $chapo[$x]);
				list($year, $month, $day, $time1) = explode("\t", date("Y\tm\td\tH:i", time() - mt_rand(7*24*3600, 3*365*24*3600)));
				$title = $liste[$src]['title'];
				if(defined('PLX_VERSION')) {
					if(version_compare(PLX_VERSION, '5.8.7', '>=')) {
						$dates = array(
							'date_publication_year'		=> $year,
							'date_publication_month'	=> $month,
							'date_publication_day'		=> $day,
							'date_publication_time'		=> $time1,
						);
					} else {
						$values = array(
							'year'				=> $year,
							'month'				=> $month,
							'day'				=> $day,
							'time'				=> $time1,
						);
						$dates = array();
						foreach(array('creation', 'publication', 'updated') as $context) {
							foreach($values as $k=>$v) {
								$dates['date_' . $context . '_' . $k] = $values[$k];
							}
						}
					}
				} else {
					# Old version of PluXml <= 5.4
					$dates = array(
						'year'				=> $year,
						'month'				=> $month,
						'day'				=> $day,
						'time'				=> $time1,
					);
				}
				$article = array_merge(
					array(
						'url'				=> $src,
						'title'				=> $title,
						'allow_com'			=> '0',
						'template'			=> !empty($template) ? $template : array_keys($aTemplates)[mt_rand(0, count($aTemplates) - 1)],
						'chapo'				=> '<p>'.implode("\n", $chapo).'</p>',
						'content'			=> implode("\n", $temp),
						'tags'				=> getTags($title, $enabledTags),
						'meta_description'	=> '',
						'meta_keywords'		=> '',
						'title_htmltag'		=> str_replace('_', ' ', substr($plxPlugin->getParam('archive'), 0, -4)),
						'moderate'			=> '',
						'publish'			=> '1',
						'catId'				=> $enabledCats ? array(addCat($liste[$src]['cat'], $enabledCats)) : '000',
						'author'			=> !empty($author) ? $author : array_keys($aAuthors)[mt_rand(0, count($aAuthors) - 1)],
						'publish'			=> '1',
					),
					$dates
				);
				// $artId = '0000';
				$artId = str_pad($nextIdArticle + $i, 4, '0', STR_PAD_LEFT);
				$plxAdmin->editArticle($article, $artId);
				unset($liste[$src]);
			} else {
				// Toutes les fables sont publiées
				plxMsg::error($plxPlugin->getLang('ALL_PUBLISHED'));
				break;
			}
		}
	}
}

if(!empty($_POST)) {
	plxToken::validateFormToken();
	// Saving parameters
	foreach ($fields as $field=>$type) {
		switch ($type) {
			case 'numeric' :
				$value = (!empty($_POST[$field])) ? intval($_POST[$field]) : '';
				break;
			case 'boolean' :
				$value = (isset($_POST[$field])) ? 1 : 0;
				break;
			default :
				$value = (!empty($_POST[$field])) ? $_POST[$field] : '';
				break;
		}
		if (empty($value) and in_array($field, array_keys($default_values))) {
			$value = $default_values[$field];
		}
		$plxPlugin->setParam($field, $value, ($type != 'boolean') ? $type : 'numeric');
	}
	$plxPlugin->saveParams();

	// publishing some articles
	readArticles(PLX_PLUGINS.$plugin.'/'.$plxPlugin->getParam('archive'), $plxPlugin->getParam('nb_pages'));

	header('Location: parametres_plugin.php?p=' . $plugin);
	exit;
}

// --- And now, the Form ! -------

if(count($aTemplates) > 1) {
	$aTemplates = array_merge(array('' => $plxPlugin->getLang('ALL_TEMPLATES')), $aTemplates);
}

$files = plxGlob::getInstance(PLX_PLUGINS . $plugin);
if ($array = $files->query('#\.zip$#')) {
	foreach($array as $k=>$v) {
		$aArchives[$v] = str_replace('_', ' ', basename($v, '.zip'));
	}
}

if(count($aAuthors) > 1) {
	$aAuthors = array_merge(array('' => $plxPlugin->getLang('EVERYBODY')), $aAuthors);
}

?>
		<h2><?= $plxPlugin->getInfo('title') ?></h2>
<?php
if (!class_exists('ZipArchive')) {
?>
	<h4 class="lesFablesWarning"><?php $plxPlugin->lang('ZIP_ARCHIVE_MISSING'); ?></h4>
<?php
} else {
?>
		<form id="form_<?= $plugin; ?>" method="post">
			<?= plxToken::getTokenPostMethod() ?>
<?php
foreach ($fields as $field=>$type) {
	$value = $plxPlugin->getParam($field);
	if (empty($value) && array_key_exists($field, $default_values))
		$value = $default_values[$field];
?>
				<label>
					<span><?php $plxPlugin->lang(strtoupper($field)); ?></span>
<?php
	switch ($type) {
		case 'numeric' :
?>
				<input type="number" name="<?= $field ?>" value="<?= $value ?>" min-="2" max="50" />
<?php
			break;
		case 'boolean' :
				$checked = ($value) ? ' checked' : '' ?>
				<input type="checkbox" name="<?= $field; ?>" value="1"<?= $checked; ?> />
<?php
			break;
		default :
			switch ($field) {
				case 'template' :
					plxUtils::printSelect($field, $aTemplates, $value);
					break;
				case 'archive' :
					plxUtils::printSelect($field, $aArchives, $value);
					break;
				case 'author' :
					plxUtils::printSelect($field, $aAuthors, $value);
					break;
				default :
?>
				<input type="text" name="<?= $field ?>" value="<?= $value ?>" />
<?php
					break;
			}
			break;
	}
?>
				</label>
<?php
}
?>
			<p>
				<?php $plxPlugin->lang('INFO'); ?>
			</p><p>
				<input type="submit" value="<?php $plxPlugin->lang('GENERATE_ARTICLES'); ?>" />
			</p>
		</form>
<?php
}
